%  Get Cell density from spots finding function
%  It finds the spots from the first channel (vChannelIndex = 0)
%  It finds the area under these spots found
%  It outputs the cell density number into excel file named results.xlsx
%  It also produces a *.png image with the selected region for area
%  calculation
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Spots Functions">
%        <Item name="Get Cell Density v3" icon="Matlab">
%          <Command>MatlabXT::getCellDensity3(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%    </CustomTools>
%
%
%  Description:
%
%

function getCellDensity3(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
    vImarisApplication = aImarisApplicationID;
end

% ------------------------------------------------------
% The following MATLAB code detects spots for the selected channel...
vDataSet = vImarisApplication.GetDataSet; %% Current dataset

% Find the Nucleous (blue spots)
vChannelIndex = 0;
vSpotDiameter = 6;
vBackgroundSubtraction = 1;
vSpotValues = vImarisApplication.GetImageProcessing.DetectSpots2( ...
    vDataSet,[],vChannelIndex,vSpotDiameter,vBackgroundSubtraction,'"Quality" above 3.2');
xyz = vSpotValues.GetPositionsXYZ;
vPositionsXYZnucOri = xyz;
vIndicesTnuc = vSpotValues.GetIndicesT;
vRadiiNuc = vSpotValues.GetRadii;

% Get the Xmax and Ymax values (the Xmin and Ymin values are 0)
Xmax = vDataSet.GetExtendMaxX;
Ymax = vDataSet.GetExtendMaxY;


% add the spots to the Imaris Scene
% create the bleu spots
vSpotsNuc = vImarisApplication.GetFactory.CreateSpots;
vSpotsNuc.SetColorRGBA(rgb2rgba(0,0,255));
vSpotsNuc.Set(vPositionsXYZnucOri,vIndicesTnuc,vRadiiNuc);
vSpotsNuc.SetName('Nucleous');

vSurpassScene = vImarisApplication.GetSurpassScene;
vSurpassScene.AddChild(vSpotsNuc,-1);
vImarisApplication.SetSurpassSelection(vSpotsNuc);

spotValuesNuc = vSpotsNuc.Get();
xyzNuc = double(spotValuesNuc.mPositionsXYZ);

% select the nucleous close to the border
xyz = round(xyzNuc);

% make the xy positions as integer values to fit the Matlab Image
images = double(vDataSet.GetDataSliceFloats(0,0,0));
xyz(:,1) = xyz(:,1)./Xmax.*size(images,1);
xyz(:,2) = xyz(:,2)./Ymax.*size(images,2);
vFileName = vImarisApplication.GetCurrentFileName;
nname = char(vFileName);
iname = strrep(nname,'.ims','.png');
[~,s] = groupPerimeter(xyz,1.2,size(images,1),size(images,2),iname);


dX = Xmax/size(images,1);
dY = Ymax/size(images,2);
density = size(xyz,1)/(s*dX*dY);

% write the density data

vFileName = {char(vFileName)};
nnameInd = strfind(nname,'\');
if isempty(nnameInd)
    nnameInd = strfind(nname,'/');
end
nname = strcat(nname(1:nnameInd(end)),'results.xlsx');
if exist(nname,'file') == 2
    [a,b] = xlsread(nname);
else
    a = [];
    b = [];
end
c(:,1) = [b;repmat({char(vFileName)},[size(density,2) 1])];
c(:,2) = num2cell([a;density']);
xlswrite(strcat(nname(1:nnameInd(end)),'results.xlsx'),c);


end

function [sind,s] = groupPerimeter(xyz,spacingFactor,xs,ys,iname)

xyz = double(xyz);
D = calculateDistances(xyz);
spacingFactor = round(mean(min(D))*spacingFactor);
se = strel('disk',spacingFactor);

% create a dilated shape of the points to surround the object
xyr = round(xyz(:,1:2));
np = false(ys+2*spacingFactor,xs+2*spacingFactor);
for i = 1:size(xyr,1)
    np(xyr(i,2)+spacingFactor,xyr(i,1)+spacingFactor) = true;
end
np = imdilate(np,se);
np = imfill(np,'holes');
props = regionprops(np,'Area','Centroid');
[~,big] = sort([props.Area],'descend');
np = bwlabel(np) == big(1);
np = np(end:-1:1,:);
imwrite(np,iname);

s = sum(np(:));

xy = pcf(np);
% find the closest points to that shape
[~,ind] = closestpoint(xyr',xy');
sind = unique(ind);


end

% function to set the colors

 function rgba = rgb2rgba(R,G,B)
        
        rgba = bin2dec(sprintf('%08d%08d%08d',str2double(dec2bin(B)),...
            str2double(dec2bin(G)),str2double(dec2bin(R))));
 end

% function to calculate the distances

function D = calculateDistances(xyz)

    D = zeros(size(xyz,1));
    for ii = 1:size(xyz,1)
        p1 = repmat(xyz(ii,:),[size(xyz,1) 1]);
        D(ii,:) = sqrt(sum((xyz - p1).^2,2));
    end
    di = logical(diag(ones(size(D,1),1)));
    D(di) = nan;

end

% function to find the contour of the object
function xy = pcf(bw)

    bwt = bw;
    bw = false(size(bwt)+2);
    bw(2:end-1,2:end-1) = bwt;
    [y,x] = find(bw);
    % --------------------------------------------------
    % Initialize variables
    m(1,:) = [0 1];
    x = x(1);
    y = y(1);
    i = 1;
    % --------------------------------------------------
    % Start the procedure to find the contour until the number of left/right
    % steps AND number of up/down steps are equals.
    while (sum(m(:,1)) ~=0) || (sum(m(:,2)) ~=0)
        % --------------------------------------------------
        % enter the coordinate of inner contour pixel
        xy_in(i,:) = [x y] - 1; %#ok<AGROW>
        % --------------------------------------------------
        % find the coordinate of the diagonal pixel (xt,yt) and the coordinate
        % of the outter pixel (x_out,y_out)
        if abs(m(i,2))
            xt = x - m(i,2);
            x_out = xt;
            yt = y + m(i,2);
            y_out = y;
        else
            xt = x + m(i,1);
            x_out = x;
            yt = y + m(i,1);
            y_out = yt;
        end
        % --------------------------------------------------
        % enter the coordinate of outter contour pixel
        xy_out(i,:) = [x_out y_out] - 1; %#ok<AGROW>
        % --------------------------------------------------
        % check if the diagonal pixel is part of the object or not
        if bw(yt,xt)
            m(i+1,:) = [-m(i,2) m(i,1)];
            x = xt;
            y = yt;
        else
            % --------------------------------------------------
            % find the coordinate of the pixel in front of the current pixel.
            xt = x + m(i,1);
            yt = y + m(i,2);
            % --------------------------------------------------
            % check if the pixel in front is part of the object or not.
            if bw(yt,xt)
                x = xt;
                y = yt;
                m(i+1,:) = [m(i,1) m(i,2)];
            else
                m(i+1,:) = [m(i,2) -m(i,1)];
            end
        end
        i = i + 1;
    end
    xy = (xy_in+xy_out)/2;
end

% function to find the closest points
function [dist,ind] = closestpoint(small,big)

    dist = zeros(1,size(big,2));
    ind = zeros(1,size(big,2));

    for i = 1:size(big,2)
        r = repmat(big(:,i),[1 size(small,2)]);
        alldist = sqrt(sum((r-small).^2,1));
        [dist(i),ind(i)] = min(alldist);
    end
end
